# Ember-C3 

[![npm version](https://badge.fury.io/js/ember-c3.svg)](http://badge.fury.io/js/ember-c3)
[![Join the chat at https://gitter.im/Glavin001/ember-c3](https://badges.gitter.im/Join%20Chat.svg)](https://gitter.im/Glavin001/ember-c3?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)
 
> Ember component library for [C3, a D3-based reusable chart library](https://github.com/masayuki0812/c3).

**Live Demo:** http://glavin001.github.io/ember-c3/

Alternatively take a look at [Ember-NVD3](https://github.com/Glavin001/ember-nvd3) for your charting needs.

## Ember Versions

This addon now uses ember-cli-babel version 7 which requires ember-cli 2.13 and above

---

Installation
------------------------------------------------------------------------------

```bash
ember install ember-c3
```

## Usage

For a complete set of examples, see the [dummy app in `tests/dummy/app/`](https://github.com/Glavin001/ember-c3/tree/master/tests/dummy/app)

<!-- (https://github.com/Glavin001/ember-c3/tree/master/tests/dummy/app). -->

| Combination |
| --- |
| ![image](https://cloud.githubusercontent.com/assets/1885333/9792404/4e750a06-57b6-11e5-96c1-8a6cabdef4ac.png) |

| Timeseries |
| --- |
| ![ember-c3-timeseries-4](https://cloud.githubusercontent.com/assets/1885333/9792600/6032061c-57b7-11e5-9a2c-80e680d274a5.gif) |

| Gauge | Pie | Donut |
| --- | --- | --- |
| ![ember-c3-gauge-2](https://cloud.githubusercontent.com/assets/1885333/9792503/c7ebc05a-57b6-11e5-8d43-116e9dd7c368.gif) | ![ember-c3-pie-1](https://cloud.githubusercontent.com/assets/1885333/9792639/8f90f5ee-57b7-11e5-859c-9a083255df4a.gif) | ![ember-c3-donut-1](https://cloud.githubusercontent.com/assets/1885333/9792665/ad4a6ee4-57b7-11e5-90d4-df152b24e4a3.gif)|

### Basic

Where `model` is your C3 data:

```handlebars
{{c3-chart data=model}}
```

### Advanced

See http://c3js.org/examples.html for examples of how to use C3.

```handlebars
{{c3-chart
  c3chart=chart
  data=model
  axis=axis
  regions=regions
  bar=bar
  pie=pie
  donut=donut
  gauge=gauge
  line=line
  area=area
  grid=grid
  legend=legend
  tooltip=tooltip
  interaction=interaction
  subchart=subchart
  zoom=zoom
  point=point
  size=size
  padding=padding
  title=title
  color=color
  dtitle=dtitle
  transition=transition
  unloadDataBeforeChange=true
  oninit
  onrendered
  onmouseover
  onmouseout
  onresize
  onresized
}}
```

### Component Properties
The properties match the corresponding C3 objects found in the [C3 Documentation](https://c3js.org/gettingstarted.html#generate).  As documented, most C3 settings (i.e. bar, axis, size, etc) can be included in the data object.

The component properties break out the settings to simplify chart configuration.  Note: The chart type is **always** assigned in the chart data object.

The properties with an asterisk are the only ones updated on the chart when a property change is triggered when `notifyPropertyChange` is called.  See examples in the dummy app.

Property | Description | Example
---------|-------------|--------
c3chart | Points to the C3 chart generated by the component.  Any C3 api method can be executed using this property.  See the methods section below, chart object and drill down examples | chart.hide("data1")
  data* | C3 data [object](https://c3js.org/gettingstarted.html#generate). Chart data is mutable after the chart is created
  axis* | C3 axis [object](https://c3js.org/reference.html#axis-rotated). See C3 examples for combining with data object.  Chart axis are mutable after the chart is created
  regions | need to test may need to be with data
  bar | Used to assign bar chart [properties](https://c3js.org/reference.html#bar-width) |
  pie | Used to assign pie chart [properties](https://c3js.org/reference.html#pie-label-show) |
  donut | Used to assign donut chart [properties](https://c3js.org/reference.html#donut-label-show) |
  gauge | Used to assign gauge chart [properties](https://c3js.org/reference.html#gauge-label-show) |
  line | Used to assign line chart [properties](https://c3js.org/reference.html#line-connectNull) |
  area | Used to assign area chart [properties](https://c3js.org/reference.html#area-zerobased) |
  point | Used to assign data point [properties](https://c3js.org/reference.html#point-show) |
  grid | Used to show, hide and modify the graph grid.  See [docs](https://c3js.org/reference.html#grid-x-show)
  legend | Show, hide and modify the legend position.  See [docs](https://c3js.org/reference.html#legend-show)
  tooltip | Show, hide and modify the tooltip.  See [docs](https://c3js.org/reference.html#tooltip-show)
  interaction | Enable or disable [interactions](https://c3js.org/reference.html#interaction-enabled) | interaction: { enabled: false }
  subchart | Show, hide and modify C3 sub charts.  See [docs](https://c3js.org/reference.html#subchart-show)
  zoom | Set C3 zoom features. See [docs](https://c3js.org/reference.html#zoom-enabled)
  size | Control chart size see [docs](https://c3js.org/reference.html#size-width) | size: {width: 640 }
  padding | Set padding around graph.  See docs(https://c3js.org/reference.html#padding-top)  | padding: { top: 20}
  title | Set chart title | title: { text: "This is my chart" }
  color* | Used to assign color [properties](https://c3js.org/reference.html#color-pattern). The chart colors are mutable after chart creation
  dtitle | Use to dynamically change the chart title, which isn't supported natively by C3. See details below | {text: "My New Title", refresh: false}
  transition | Equivalent to [transition.duration](https://c3js.org/reference.html#transition-duration).  Default duration is 350ms.  Transition times less than 300ms may not render properly.  Use chart.load() and .unload() if shorter times are required
  unloadDataBeforeChange | When set to true the data will be unloaded before new data is loaded with didUpdateAttrs().  Useful for pie and donut chart data changes.  Also do data changes with .load() and .unload()
  oninit | C3 chart event, see events section below | oninit=(action "init")
  onrendered | C3 chart event, see events section below | onrendered=(action "render")
  onmouseover | C3 chart event, see events section below | onmouseover=(action "mouseover")
  onmouseout | C3 chart event, see events section below | onmouseout=(action "mouseout")
  onresize | C3 chart event, see events section below | onresize=(action "resize")
  onresized | C3 chart event, see events section below | onresized=(action "resized")

### dtitle
The `dtitle` property is used to dynamically change a chart's title.  C3 doesn't natively support this without forcing a chart redraw (.flush()) which can cause side effects. This especially true if the graph data is being dynamically changed using the api.

The title can be set using the `.c3-title` class but that doesn't provide abstraction from c3 internals.

`dtitle` gives you some control over side effects using a parameter to control how the graph is refreshed.  An object is passed into `dtitle` and the second parameter `refresh` indicates whether all properties should be refreshed or only the chart title.  Setting `refresh` to false will only refresh the title and ignore changes to the data, colors and axis properties.  A short example is below.  See the drill down example to see how `dttile` is used and potential side effects.

The chart's initial title is set using the `title` parameter.  

```handlebars
{{c3-chart data=data title=title dtitle=dtitle}}
```

```javascript
import Controller from "@ember/controller";
/* eslint ember/avoid-leaking-state-in-ember-objects: "off" */

export default Controller.extend({

  tile: { text: "Orignal title" },

  actions: {
    changeTitle() {
      this.set("dtitle", { text: "Updated title", refresh: false });
    }
  }
})
```

### C3 Methods
If you assign a controller property to the c3chart property, you can use most of C3's api [methods](https://c3js.org/reference.html#api-focus).  Not all the methods have been tested.

templates/my-route.hbs
```handlebars
{{c3-chart data=mydata c3chart=chart}}

<button onclick={{action "loadUS"}}>US Cars</button>
<button onclikc={{action "loadGerman"}}>German Cars</button>
<button onclikc={{action "resetData"}}>Reset</button>
```

controllers/my-route.js
```javascript
import { later } from "@ember/runloop";
import Controller from "@ember/controller";
/* eslint ember/avoid-leaking-state-in-ember-objects: "off" */

export default Controller.extend({
 chart: null,

 baseData:   
    { 
      columns: [
        ["US", 64],
        ["German", 36]
      ],
      type: "donut"
    },
 
 modelsGerman: [
        ["Mercedes", 12],
        ["Volkswagon", 54],
        ["BMW", 34]
      ],

  modelsUS: [
    ["Ford", 35],
    ["Chevy", 26],
    ["Tesla", 2],
    ["Buick", 10],
    ["Dodge", 27]
  ],

  actions: {
     resetData() {
      this.chart.load({ columns: this.baseData });
      this.chart.unload("Mercedes", "Volkswagon", 
      "BMW", "Ford", "Chevy", "Tesla", "Buick", "Dodge");
    },

    loadUS() {
      this.chart.load({ columns: this.modelsUS});
      this.chart.unload("US", "German");
    },
       
    loadGerman() {
      this.chart.load({ columns: this.modelsGerman});
      this.chart.unload("US", "German");
    }
  }
});
```

### C3 Events
C3 emits two types of events - [chart](https://c3js.org/reference.html#oninit) and [data](https://c3js.org/reference.html#data-onclick) events.  Chart events can be assigned to a closure action via a property.  Data events **must** be assigned an action in the data object.

For connivence, the chart object is passed, with the exception of _oninit_ to chart events.  An example of a chart and data event is shown below.  Note the use of `bind` for tying actions to data events. See the dummy app for more examples

templates/application.hbs
```handlebars
{{c3-chart 
  data=data
  oninit=(action 'init')
  }}
```
controllers/application.js
```javascript
import Controller from "@ember/controller";
import { computed } from "@ember/object";
import { bind } from "@ember/runloop";

export default Controller.extend({
 data: computed(function() {
    // iris data from R
    return {
      columns: [
        ["data1", 30],
        ["data2", 120],
        ["data3", 10],
        ["data4", 45],
        ["data5", 90]
      ],
      type: "pie",
      onclick: bind(this, this.actions.onClick)
    };
  }),

 actions: {
    // oninit chart event
    init(chart){
      console.log("chart inited")
    },

    // data event - triggered when data point clicked
    onClick(d, i) {
      alert(`Data ${d.name} has a value of ${d.value}`)
    },
  }
});
```
### Accessing D3

You can use the D3 library in your application by importing it where needed

```javascript
import d3 from "d3";
```

### Helpful Links

- [Extending ember-c3 charts using base-component approach](https://medium.com/nimbo-x-development/extending-ember-c3-charts-using-base-component-approach-78df1d635050#.k8j8gnnqa)

## Development

* `git clone` this repository
* `npm install` or `yarn install`
* `ember server`
* Visit your app at [http://localhost:4200](http://localhost:4200).
* `npm run lint:hbs`
* `npm run lint:js`
* `npm run lint:js -- --fix`

### Running tests

* `ember test` – Runs the test suite on the current Ember version
* `ember test --server` – Runs the test suite in "watch mode"
* `ember try:each` – Runs the test suite against multiple Ember versions

Note: C3 chart events `onresize` and `onresized` are not tested

For more information on using ember-cli, visit [https://ember-cli.com/](https://ember-cli.com/).

### Dummy app on gh-pages
Use `npm run deploy` to build and deploy dummy app to the gh-pages branch using [ember-cli-github-pages](https://github.com/poetic/ember-cli-github-pages)

License
------------------------------------------------------------------------------

This project is licensed under the [MIT License](LICENSE.md).
